"""
High-level tests for the pyphonebook CLI.

"""

EXPECTED_DH1=[
    "Cognome1                 Dh1             61111 62901 162111 IT  -   -      91 2-027",
]
EXPECTED_USER4=[
    "Vastname4                User            -     -     161135 BE  CSS SET   799 2-034",
]
TERSE_WARNING=[
    "#-------------------------------------------------------------------------------",
    "WARNING! If you built a script on top of the PhoneBook CLI, please consider",
    "using the PhoneBook Python API instead.",
    "#-------------------------------------------------------------------------------",
]

def test_no_args(pyphonebook):
    ret = pyphonebook.command()
    assert ret.returncode == 1
    assert ret.stdout.startswith('usage: pyphonebook')
    assert ret.stderr == ''

def test_usage(pyphonebook):
    ret = pyphonebook.command('--help')
    assert ret.success
    assert ret.stdout.startswith('usage: pyphonebook')
    assert ret.stderr == ''

def test_verboseonly(pyphonebook):
    ret = pyphonebook.command('-v')
    assert ret.returncode == 1
    assert ret.stdout.startswith('usage: pyphonebook')
    assert ret.stderr == ''

def test_terseonly(pyphonebook):
    ret = pyphonebook.command('-terse login')
    assert ret.returncode == 1
    assert ret.stdout.startswith('usage: pyphonebook')
    assert ret.stderr == ''

def test_jsononly(pyphonebook):
    ret = pyphonebook.command('-j login')
    assert ret.returncode == 1
    assert ret.stdout.startswith('usage: pyphonebook')
    assert ret.stderr == ''

def test_single_user(pyphonebook):
    # Find by login name
    pyphonebook.check('dcognome1', EXPECTED_DH1)
    # Find by last name, including partial matches (and mixed case)
    pyphonebook.check('Cognome1', EXPECTED_DH1)
    pyphonebook.check('Cognome', EXPECTED_DH1)
    pyphonebook.check('COGNO', EXPECTED_DH1)
    pyphonebook.check('nome', EXPECTED_DH1)
    # Find by first name
    pyphonebook.check('DH1', EXPECTED_DH1)
    # Find by full name
    pyphonebook.check('Dh1 Cognome1', EXPECTED_DH1)
    # Find by phone number
    pyphonebook.check('61111', EXPECTED_DH1)
    pyphonebook.check('162111', EXPECTED_DH1)
    pyphonebook.check('+41227661111', EXPECTED_DH1)

def test_no_match(pyphonebook):
    pyphonebook.check('nobody', returncode = 1)

def test_multiple_users_unsorted(pyphonebook):
    ret = pyphonebook.command('user1')
    assert ret.success
    assert ret.stderr == ''
    assert ret.stdout.count('\n') == 2
    assert "Lastname1                User " in ret.stdout
    assert "User1                    esi  " in ret.stdout

def test_multiple_users_sorted(pyphonebook):
    EXPECTED=[
        "Lastname1                User            72222 71111 161234 IT  CM  LCS    99 1-010", # 0
        "van Lastname3            User            -     -     -      IT  CM  COG    99 1-005", # 1
        "Vastname4                User            -     -     161135 BE  CSS SET   799 2-034", # 2
        "Lastname5                User            -     -     -      Oth -   -             -", # 3
        "Lastname6                User            71153 -     164658 IT  CM  -      99 R-008", # 4
    ]
    pyphonebook.check('astname --orderby login', EXPECTED)
    pyphonebook.check('astname --orderby surname', [EXPECTED[x] for x in (0, 3, 4, 1, 2)])
    pyphonebook.check('astname --orderby firstname --orderby surname', [EXPECTED[x] for x in (0, 3, 4, 1, 2)])
    pyphonebook.check('astname --orderby department', [EXPECTED[x] for x in (2, 0, 1, 4, 3)])
    pyphonebook.check('astname --orderby group', [EXPECTED[x] for x in (3, 0, 1, 4, 2)])
    pyphonebook.check('astname --orderby cerngroup', [EXPECTED[x] for x in (3, 0, 1, 4, 2)])
    pyphonebook.check('astname --orderby cernsection', [EXPECTED[x] for x in (3, 4, 1, 0, 2)])
    pyphonebook.check('astname --orderby displayname', EXPECTED)

    # Unknown order by fields return errors
    pyphonebook.check('user1 --orderby kk',
        stderr = ["Error: Unrecognized -order(by) option value (kk), see --help for help."],
        returncode = 2
    )

def test_search_surname(pyphonebook):
    pyphonebook.check('--surname cognome1', EXPECTED_DH1)
    pyphonebook.check('--surname COGNOME1', EXPECTED_DH1)

def test_search_firstname(pyphonebook):
    pyphonebook.check('--firstname dh1', EXPECTED_DH1)
    pyphonebook.check('--firstname DH1', EXPECTED_DH1)

def test_search_building(pyphonebook):
    pyphonebook.check('--building 91', EXPECTED_DH1)

def test_search_office(pyphonebook):
    pyphonebook.check('--office 2-027', EXPECTED_DH1)

def test_search_email(pyphonebook):
    pyphonebook.check('--email dcognome1@cern.ch', EXPECTED_DH1)

def test_search_login(pyphonebook):
    pyphonebook.check('--login dcognome1', EXPECTED_DH1)

def test_search_department(pyphonebook):
    pyphonebook.check('--department BE', EXPECTED_USER4)

def test_search_group(pyphonebook):
    pyphonebook.check('--group CSS', EXPECTED_USER4)

def test_search_section(pyphonebook):
    pyphonebook.check('--section SET', EXPECTED_USER4)

def test_search_phone(pyphonebook):
    pyphonebook.check('--phone 61111', EXPECTED_DH1)
    pyphonebook.check('--phone +41227661111', EXPECTED_DH1)
    pyphonebook.check('--phone 41227661111', EXPECTED_DH1)

def test_search_mobile(pyphonebook):
    pyphonebook.check('--mobile 162111', EXPECTED_DH1)
    pyphonebook.check('--mobile +41754112111', EXPECTED_DH1)
    pyphonebook.check('--mobile 41754112111', EXPECTED_DH1)

def test_search_pobox(pyphonebook):
    pyphonebook.check('--pobox M33900', EXPECTED_DH1)
    pyphonebook.check('--pobox m33900', EXPECTED_DH1)

def test_search_ccid(pyphonebook):
    pyphonebook.check('--ccid 373457', EXPECTED_DH1)

def test_search_uid(pyphonebook):
    pyphonebook.check('--uid 140840', EXPECTED_DH1)

def test_search_gid(pyphonebook):
    EXPECTED=[
        "Cognome1                 Dh1             61111 62901 162111 IT  -   -      91 2-027",
        "Lastname6                User            71153 -     164658 IT  CM  -      99 R-008",
    ]
    pyphonebook.check('--gid 1025', EXPECTED)

def test_search_homedir(pyphonebook):
    pyphonebook.check('--homedir /afs/cern.ch/user/d/dcognome1', EXPECTED_DH1)
    pyphonebook.check('--homedir /*/dcognome1', EXPECTED_DH1)

def test_output_terse(pyphonebook):
    pyphonebook.check('--login dcognome1 --terse firstname --terse surname --terse login',
        stdout = ["Dh1;Cognome1;dcognome1;"],
        stderr = TERSE_WARNING)
    pyphonebook.check('--login dcognome1 --terse login --terse mobile --terse pobox',
        stdout = ["dcognome1;+41754112111;M33900;"],
        stderr = TERSE_WARNING)
    pyphonebook.check('--login dcognome1 --terse email',
        stdout = ["dh1.cognome1@cern.ch;"],
        stderr = TERSE_WARNING)

    # terse output is unsorted by default
    pyphonebook.check('user1 --terse login --terse mobile --terse pobox --orderby login',
        stdout = [
            "esiuser2;-;-;",
            "secondary1;+41754111234;M35110;",
            "user1;+41754111234;M35110;",
        ],
        stderr = TERSE_WARNING)

    # Error on invalid field names
    pyphonebook.check('--login dcognome1 --terse LOGIN',
        stderr = ["Error: Unrecognized -t(erse) option value (LOGIN), see --help for help."],
        returncode = 2)

def test_shortargs(pyphonebook):
    pyphonebook.check('--login dcognome1', EXPECTED_DH1)
    pyphonebook.check('--logi dcognome1', EXPECTED_DH1)
    pyphonebook.check('--log dcognome1', EXPECTED_DH1)
    pyphonebook.check('--lo dcognome1', EXPECTED_DH1)
    pyphonebook.check('--l dcognome1', EXPECTED_DH1)
    pyphonebook.check('-login dcognome1', EXPECTED_DH1)
    pyphonebook.check('-logi dcognome1', EXPECTED_DH1)
    pyphonebook.check('-log dcognome1', EXPECTED_DH1)
    pyphonebook.check('-lo dcognome1', EXPECTED_DH1)
    pyphonebook.check('-l dcognome1', EXPECTED_DH1)

def test_ambiguity(pyphonebook):
    # Ambitious options are dropped and the next string is used as a search string
    pyphonebook.check('-p M33900',
        stdout = [],
        stderr = ['Option p is ambiguous (phone, pobox)'],
        returncode = 1
    )

    # This one does find something because "cog" is matched as a lastname
    pyphonebook.check('-s cog',
        stdout = EXPECTED_DH1,
        stderr = ['Option s is ambiguous (surname, section)']
    )

def test_full_output(pyphonebook):
    EXPECTED_DH1_FULL = [
        "#------------------------------------------------------------------------------",
        "Surname:            Cognome1",
        "Firstname:          Dh1",
        "Display Name:       Dh1 Cognome1",
        "E-mail:             dh1.cognome1@cern.ch",
        "External E-mail:    dh1.cognome1.roma@gmail.com",
        "Telephone:          +41227661111                              (internal:  61111)",
        "Other telephone:    +41227662901                              (internal:  62901)",
        "Mobile:             +41754112111                              (internal: 162111)",
        "Facsimile:          -                                         (internal:      -)",
        "Department:         IT",
        "Group:              -",
        "Section:            -",
        "POBox:              M33900                                    (internal mail)",
        "Bld. Floor-Room:    91 2-027",
        "Organization:       CERN",
        "Computer Center ID: 373457",
        "Low user ID:        -",
        "",
        "Computer account(s):",
        "Login    Grp    St Uid    Gid  Last login     Shell      Home directory",
        "",
        "dcognome g2     PA 140840 1025 23/08/21 05:33 /bin/bash  /afs/cern.ch/user/d/dcognome1",
        "#-------------------------------------------------------------------------------",
        "#Account St(atus): P(rimary), S(econdary), s(ervice), U(nknown)",
        "#                  A(ctive),  D(isabled),  P(assword expired),  L(ocked out)",
    ]
    pyphonebook.check('--login dcognome1 --all', EXPECTED_DH1_FULL)

    EXPECTED_U3_FULL = [
        "#------------------------------------------------------------------------------",
        "Surname:            van Lastname3",
        "Firstname:          User",
        "Display Name:       User3 van Lastname3",
        "E-mail:             user3.Lastname3@cern.ch",
        "External E-mail:    user3@gmail.com",
        "Telephone:          -                                         (internal:      -)",
        "Mobile:             -                                         (internal:      -)",
        "Facsimile:          +41227661123                              (internal:  61123)",
        "Department:         IT",
        "Group:              CM",
        "Section:            COG",
        "POBox:              M35320                                    (internal mail)",
        "Bld. Floor-Room:    99 1-005",
        "Organization:       CERN",
        "Computer Center ID: 565129",
        "Low user ID:        -",
        "",
        "Computer account(s):",
        "Login    Grp    St Uid    Gid  Last login     Shell      Home directory",
        "",
        "user3    g1     PA 36030  1024 27/08/21 15:18 /bin/bash  /afs/cern.ch/user/u/user3",
        "#-------------------------------------------------------------------------------",
        "#Account St(atus): P(rimary), S(econdary), s(ervice), U(nknown)",
        "#                  A(ctive),  D(isabled),  P(assword expired),  L(ocked out)",
    ]
    pyphonebook.check('--login user3 --all', EXPECTED_U3_FULL)

def test_output_json(pyphonebook):
    pyphonebook.check_json('--login dcognome1 --json firstname --json surname --json login',
        stdout_json='[{"firstname": ["Dh1"], "surname": ["Cognome1"], "login": ["dcognome1"]}]'
    )
    pyphonebook.check_json('--login dcognome1 --json email',
        stdout_json='[{"email": ["dh1.cognome1@cern.ch"]}]'
    )

def test_output_json_multiple(pyphonebook):
    pyphonebook.check_json('user1 --json login --json mobile --json pobox',
        stdout_json = '[ {"login": ["esiuser2"], "mobile": [], "pobox": []},' + \
            '{"login": ["secondary1"], "mobile": ["+41754111234"], "pobox": ["M35110"]},' + \
            '{"login": ["user1"], "mobile": ["+41754111234"], "pobox": ["M35110"]} ]'
    )

def test_output_json_nobody(pyphonebook):
    pyphonebook.check_json('nobody --json login --json mobile --json pobox',
        stdout_json = '[]',
        returncode = 1
    )

def test_output_json_bad_option(pyphonebook):
    pyphonebook.check('nobody --json login --json birthday',
        stderr = ['Error: Unrecognized -j(son) option value (birthday), see --help for help.'],
        returncode = 2
    )
