"""
High-level tests for the PhoneBook class.

"""
from pyphonebook import PhoneBookEntry

# TODO functions like merge_by_ccid(), arbitrary_search() and build_arbitrary_query() are not tested yet


def test_phonebook_raw_search(phonebook):
    results = phonebook._raw_search("(cn=user1)")
    assert len(results) == 1


def test_phonebook_search_raw_false(phonebook):
    results = phonebook.search(query="(cn=user1)", raw=False)
    [print(r) for r in results]
    assert len(results) == 1
    assert isinstance(results[0], PhoneBookEntry)


def test_phonebook_search_raw_true(phonebook):
    results = phonebook.search(query="(cn=user1)", raw=True)
    assert len(results) == 1
    assert isinstance(results[0], tuple)


def test_phonebook_search_by_ldap_parameter(phonebook):
    results = phonebook.search_by_ldap_parameter("sn", "van Lastname3")
    assert len(results) == 1
    assert results[0].login_name[0] == "user3"
    assert results[0].last_name[0] == "van Lastname3"


def test_phonebook_compound_search(phonebook):
    params = {"department": "BE", "preferredLanguage": "EN"}
    results = phonebook.multi_field_search(params)
    assert len(results) == 1
    assert results[0].login_name[0] == "user4"

    params = {"department": "IT", "preferredLanguage": "FR"}
    results = phonebook.multi_field_search(params)
    assert len(results) == 1
    assert results[0].login_name[0] == "user3"


def test_phonebook_validate_login_name(phonebook):
    assert phonebook.validate_login_name("user1")
    assert not phonebook.validate_login_name("user50")


def test_phonebook_search_by_login_name(phonebook):
    results = phonebook.search_by_login_name("user1")
    assert len(results) == 1
    assert results[0].login_name[0] == "user1"

    results = phonebook.search_by_login_name("user50")
    assert len(results) == 0


def test_phonebook_search_by_full_name(phonebook):
    results = phonebook.search_by_full_name("User3 van Lastname3")
    assert len(results) == 1
    assert results[0].login_name[0] == "user3"

    results = phonebook.search_by_login_name("User Five")
    assert len(results) == 0


def test_phonebook_search_by_first_name(phonebook):
    results = phonebook.search_by_first_name("User")
    assert len(results) == 5
    logins = [x.login_name[0] for x in results]
    for u in ['user1', 'user3', 'user4', 'user5', 'user6']:
        assert u in logins

    results = phonebook.search_by_first_name("Another")
    assert len(results) == 0


def test_phonebook_search_by_last_name(phonebook):
    results = phonebook.search_by_last_name("van Lastname3")
    assert len(results) == 1
    assert results[0].login_name[0] == "user3"

    results = phonebook.search_by_last_name("Five")
    assert len(results) == 0


def test_phonebook_search_by_email(phonebook):
    results = phonebook.search_by_email("user3.lastname3@cern.ch")
    assert len(results) == 1
    assert results[0].login_name[0] == "user3"

    results = phonebook.search_by_email("user.five@cern.ch")
    assert len(results) == 0


def test_phonebook_search_by_phone_number(phonebook):
    results = phonebook.search_by_phone_number("+41227671153")
    assert len(results) == 1
    assert results[0].login_name[0] == "user6"

    results = phonebook.search_by_phone_number("71153")
    assert len(results) == 1
    assert results[0].login_name[0] == "user6"

    results = phonebook.search_by_phone_number("0000000")
    assert len(results) == 0


def test_phonebook_search_by_landline(phonebook):
    results = phonebook.search_by_landline("+41227671153")
    assert len(results) == 1
    assert results[0].login_name[0] == "user6"

    results = phonebook.search_by_landline("71153")
    assert len(results) == 1
    assert results[0].login_name[0] == "user6"

    results = phonebook.search_by_landline("1234567")
    assert len(results) == 0


def test_phonebook_search_by_fax(phonebook):
    results = phonebook.search_by_fax("+41227669549")
    assert len(results) == 1
    assert results[0].login_name[0] == "user6"

    results = phonebook.search_by_fax("69549")
    assert len(results) == 1
    assert results[0].login_name[0] == "user6"

    results = phonebook.search_by_fax("0987654")
    assert len(results) == 0


def test_phonebook_search_by_mobile(phonebook):
    results = phonebook.search_by_mobile("+41754114658")
    assert len(results) == 1
    assert results[0].login_name[0] == "user6"

    results = phonebook.search_by_mobile("164658")
    assert len(results) == 1
    assert results[0].login_name[0] == "user6"

    results = phonebook.search_by_mobile("160000")
    assert len(results) == 0


def test_phonebook_search_by_post_box(phonebook):
    results = phonebook.search_by_post_box("M36010")
    assert len(results) == 1
    assert results[0].login_name[0] == "user6"

    results = phonebook.search_by_post_box("Z000001")
    assert len(results) == 0


def test_phonebook_search_by_building(phonebook):
    results = phonebook.search_by_building("99")
    assert len(results) == 3
    logins = [x.login_name[0] for x in results]
    for u in ['user1', 'user3', 'user6']:
        assert u in logins

    results = phonebook.search_by_building("111")
    assert len(results) == 0


def test_phonebook_search_by_office(phonebook):
    results = phonebook.search_by_office("010")
    assert len(results) == 1
    logins = [x.login_name[0] for x in results]
    for u in ['user1']:
        assert u in logins

    results = phonebook.search_by_office("011")
    assert len(results) == 0


def test_phonebook_search_by_department(phonebook):
    results = phonebook.search_by_department("BE")
    assert len(results) == 1
    logins = [x.login_name[0] for x in results]
    for u in ['user4']:
        assert u in logins

    results = phonebook.search_by_department("BEE")
    assert len(results) == 0


def test_phonebook_search_by_group(phonebook):
    results = phonebook.search_by_group("CSS")
    assert len(results) == 1
    assert results[0].login_name[0] == "user4"

    results = phonebook.search_by_group("BE")
    assert len(results) == 0


def test_phonebook_search_by_section(phonebook):
    results = phonebook.search_by_section("SET")
    assert len(results) == 1
    assert results[0].login_name[0] == "user4"

    results = phonebook.search_by_section("EE")
    assert len(results) == 0


def test_phonebook_search_by_organizational_unit(phonebook):
    results = phonebook.search_by_organizational_unit("BE", "CSS", "SET")
    assert len(results) == 1
    assert results[0].login_name[0] == "user4"

    results = phonebook.search_by_organizational_unit("BE", "CO", "DS")
    assert len(results) == 0


def test_phonebook_search_by_employee_id(phonebook):
    results = phonebook.search_by_employee_id("794679")
    assert len(results) == 1
    assert results[0].login_name[0] == "user4"

    results = phonebook.search_by_employee_id("000000")
    assert len(results) == 0


def test_phonebook_search_by_ccid(phonebook):
    results = phonebook.search_by_ccid("794679")
    assert len(results) == 1
    assert results[0].login_name[0] == "user4"

    results = phonebook.search_by_ccid("000000")
    assert len(results) == 0


def test_phonebook_search_by_uid(phonebook):
    results = phonebook.search_by_uid("118625")
    assert len(results) == 1
    assert results[0].login_name[0] == "user4"

    results = phonebook.search_by_uid("222222")
    assert len(results) == 0


def test_phonebook_search_by_gid(phonebook):
    results = phonebook.search_by_gid("1025")
    assert len(results) == 2
    logins = [x.login_name[0] for x in results]
    for u in ['dcognome1', 'user6']:
        assert u in logins

    results = phonebook.search_by_gid("9999")
    assert len(results) == 0


def test_phonebook_search_by_home_directory(phonebook):
    results = phonebook.search_by_home_directory("/afs/cern.ch/user/u/user1")
    assert len(results) == 1
    assert results[0].login_name[0] == "user1"

    results = phonebook.search_by_home_directory("/afs/cern.ch/user/u/user9")
    assert len(results) == 0
